/*

===== IMPORTANT =====

This is sample code demonstrating API, technology or techniques in development.
Although this sample code has been reviewed for technical accuracy, it is not
final. Apple is supplying this information to help you plan for the adoption of
the technologies and programming interfaces described herein. This information
is subject to change, and software implemented based on this sample code should
be tested with final operating system software and final documentation. Newer
versions of this sample code may be provided with future seeds of the API or
technology. For information about updates to this and other developer
documentation, view the New & Updated sidebars in subsequent documentation
seeds.

=====================

File: DateFieldCell.m
Abstract: 
Concrete subclass of EditableTableViewCell. Creates a text field within the
cell. Animates a UIDatePicker for
modifying the displayed date.


Version: 1.0

Disclaimer: IMPORTANT:  This Apple software is supplied to you by Apple Inc.
("Apple") in consideration of your agreement to the following terms, and your
use, installation, modification or redistribution of this Apple software
constitutes acceptance of these terms.  If you do not agree with these terms,
please do not use, install, modify or redistribute this Apple software.

In consideration of your agreement to abide by the following terms, and subject
to these terms, Apple grants you a personal, non-exclusive license, under
Apple's copyrights in this original Apple software (the "Apple Software"), to
use, reproduce, modify and redistribute the Apple Software, with or without
modifications, in source and/or binary forms; provided that if you redistribute
the Apple Software in its entirety and without modifications, you must retain
this notice and the following text and disclaimers in all such redistributions
of the Apple Software.
Neither the name, trademarks, service marks or logos of Apple Inc. may be used
to endorse or promote products derived from the Apple Software without specific
prior written permission from Apple.  Except as expressly stated in this notice,
no other rights or licenses, express or implied, are granted by Apple herein,
including but not limited to any patent rights that may be infringed by your
derivative works or by other works in which the Apple Software may be
incorporated.

The Apple Software is provided by Apple on an "AS IS" basis.  APPLE MAKES NO
WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND OPERATION ALONE OR IN
COMBINATION WITH YOUR PRODUCTS.

IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION, MODIFICATION AND/OR
DISTRIBUTION OF THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF
CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF
APPLE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Copyright (C) 2008 Apple Inc. All Rights Reserved.

*/

#import "DateFieldCell.h"

@implementation DateFieldCell

- (id)initWithFrame:(CGRect)frame {
    if (self = [super initWithFrame:frame]) {
        // Set the frame to CGRectZero as it will be reset in layoutSubviews
        textField = [[UITextField alloc] initWithFrame:CGRectZero];
        textField.contentVerticalAlignment = UIControlContentVerticalAlignmentCenter;
        textField.delegate = self;
        [self addSubview:textField];
        // Create a date formatter to convert the date to a string format.
        dateFormatter = [[NSDateFormatter alloc] init];
        [dateFormatter setDateStyle:NSDateFormatterMediumStyle];
        [dateFormatter setTimeStyle:NSDateFormatterNoStyle];
    }
    return self;
}

- (void)dealloc {
    // Release allocated resources.
    [textField release];
    [dateFormatter release];
    [datePicker release];
    [date release];
    [super dealloc];
}

- (void)layoutSubviews {
    // Place the subview appropriately.
    textField.frame = CGRectInset([self contentRectForBounds:[self bounds]], 10, 0);
}

// "Get" accessor for the date property.
- (NSDate *)date {
    return date;
}

// "Set" accessor for the date property, also updates the text field with the string value of the date.
- (void)setDate:(NSDate *)aDate {
    [date release];
    date = [aDate copy];
    // Update the text display.
    textField.text = [dateFormatter stringFromDate:date];
}

// Brings the date picker into view to do editing.
- (BOOL)textFieldShouldBeginEditing:(UITextField *)textField {
    BOOL shouldEdit = YES;
    // If there is a delegate and the delegate implements the appropriate method,
    // give the delegate a chance to override the action.
    if (self.delegate && [self.delegate respondsToSelector:@selector(cellShouldBeginEditing:)]) {
        shouldEdit = [self.delegate cellShouldBeginEditing:self];
    }
    if (shouldEdit) {
        // Create and configure the date picker, if necessary.
        if (!datePicker) {
            datePicker = [[UIDatePicker alloc] initWithFrame:CGRectZero];
            datePicker.datePickerMode = UIDatePickerModeDate;            
            [self.window addSubview:datePicker];
        }
        datePicker.date = date;
        // Position the date picker below the bottom of the screen.
        CGRect screenRect = [[UIScreen mainScreen] applicationFrame];
        CGSize pickerSize = [datePicker sizeThatFits:CGSizeZero];
        CGFloat screenBottom = screenRect.origin.y + screenRect.size.height;
        CGRect startRect = CGRectMake(0, screenBottom, pickerSize.width, pickerSize.height);
        datePicker.frame = startRect;
        // Calculate the position to which the picker will be animated.
        CGRect pickerRect = CGRectMake(0, screenBottom - pickerSize.height, pickerSize.width, pickerSize.height);
        // Configure the animation.
        [UIView beginAnimations:nil context:NULL];
        [UIView setAnimationDuration:0.3];
        datePicker.frame = pickerRect;
        [UIView commitAnimations];
    }
    // Always return NO to the text field because editing will never take place there.
    return NO;
}

// EditableTableViewCell override. Hides the date picker if displayed.
- (void)stopEditing {
    // If the picker has not been initialized, no action should be taken.
    if (datePicker) {
        // Retrieve the date value in the picker.
        self.date = [datePicker date];
        // Calculate the picker's final, offscreen position.
        CGRect screenRect = [[UIScreen mainScreen] applicationFrame];
        CGRect pickerRect = datePicker.frame;
        pickerRect.origin.y = screenRect.origin.y + screenRect.size.height;
        // Configure the animation.
        [UIView beginAnimations:nil context:NULL];
        [UIView setAnimationDuration:0.3];
        datePicker.frame = pickerRect;
        [UIView commitAnimations];
    }
}

@end
