/*

===== IMPORTANT =====

This is sample code demonstrating API, technology or techniques in development.
Although this sample code has been reviewed for technical accuracy, it is not
final. Apple is supplying this information to help you plan for the adoption of
the technologies and programming interfaces described herein. This information
is subject to change, and software implemented based on this sample code should
be tested with final operating system software and final documentation. Newer
versions of this sample code may be provided with future seeds of the API or
technology. For information about updates to this and other developer
documentation, view the New & Updated sidebars in subsequent documentation
seeds.

=====================

File: DetailViewController.m
Abstract: 
Displays the details of a Book object and allows the user the edit them.


Version: 1.0

Disclaimer: IMPORTANT:  This Apple software is supplied to you by Apple Inc.
("Apple") in consideration of your agreement to the following terms, and your
use, installation, modification or redistribution of this Apple software
constitutes acceptance of these terms.  If you do not agree with these terms,
please do not use, install, modify or redistribute this Apple software.

In consideration of your agreement to abide by the following terms, and subject
to these terms, Apple grants you a personal, non-exclusive license, under
Apple's copyrights in this original Apple software (the "Apple Software"), to
use, reproduce, modify and redistribute the Apple Software, with or without
modifications, in source and/or binary forms; provided that if you redistribute
the Apple Software in its entirety and without modifications, you must retain
this notice and the following text and disclaimers in all such redistributions
of the Apple Software.
Neither the name, trademarks, service marks or logos of Apple Inc. may be used
to endorse or promote products derived from the Apple Software without specific
prior written permission from Apple.  Except as expressly stated in this notice,
no other rights or licenses, express or implied, are granted by Apple herein,
including but not limited to any patent rights that may be infringed by your
derivative works or by other works in which the Apple Software may be
incorporated.

The Apple Software is provided by Apple on an "AS IS" basis.  APPLE MAKES NO
WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND OPERATION ALONE OR IN
COMBINATION WITH YOUR PRODUCTS.

IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION, MODIFICATION AND/OR
DISTRIBUTION OF THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF
CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF
APPLE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Copyright (C) 2008 Apple Inc. All Rights Reserved.

*/

#import "DetailViewController.h"
#import "Task.h"
#import "MasterViewController.h"

const CGFloat kOFFSET_FOR_KEYBOARD = 100;

// Private interface for DetailViewController - internal only methods.
@interface DetailViewController (Private)

- (void)setViewMovedUp:(BOOL)movedUp;

@end

@implementation DetailViewController

@synthesize book;

- (id)init {
    if (self = [super init]) {
        // Title displayed by the navigation controller.
        self.title = @"ToDo Detail";
    }
    return self;
}

- (void)dealloc {
    // Release allocated resources.
    [titleFieldCell release];
    [copyrightFieldCell release];
    tableView.delegate = nil;
    tableView.dataSource = nil;
    [tableView release];
    self.title = nil;
    [super dealloc];
}

- (void)loadView {
    // Create each of the cells used in the table. These cells are all of a special UITableViewCell subclass
    // designed to support inline editing.
    titleFieldCell = [[TextFieldCell alloc] initWithFrame:CGRectZero];
    titleFieldCell.delegate = self;
    copyrightFieldCell = [[DateFieldCell alloc] initWithFrame:CGRectZero];
    copyrightFieldCell.delegate = self;
    
    
    self.editing = NO;
    self.navigationItem.customRightView = self.editButton;
    // Create a table view as the main view.
    tableView = [[UITableView alloc] initWithFrame:CGRectZero style:UITableViewStyleGrouped];
    tableView.delegate = self;
    tableView.dataSource = self;
    self.view = tableView;
}

- (void)viewWillAppear:(BOOL)animated {
    // Update the view with current data before it is displayed.
    titleFieldCell.text = book.title;
    copyrightFieldCell.date = book.copyright;
    authorFieldCell.text = book.author;
    // Watch the keyboard so we can adjust the user interface if necessary.
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardWillShow:) 
            name:UIKeyboardWillShowNotification object:self.view.window]; 
}

- (void)viewWillDisappear:(BOOL)animated {
    // Unregister for keyboard notifications while not visible.
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIKeyboardWillShowNotification object:nil]; 
    // Write changes made by the user back to the object.
    book.title = titleFieldCell.text;
    book.copyright = copyrightFieldCell.date;
}

#pragma mark -
#pragma mark <EditableTableViewCellDelegate> Methods and editing management

- (BOOL)cellShouldBeginEditing:(EditableTableViewCell *)cell {
    // notify other cells to end editing
    if (![cell isEqual:titleFieldCell]) [titleFieldCell stopEditing];
    if (![cell isEqual:copyrightFieldCell]) [copyrightFieldCell stopEditing];
    if (![cell isEqual:authorFieldCell]) [authorFieldCell stopEditing];
    return self.editing;
}

- (void)cellDidEndEditing:(EditableTableViewCell *)cell {
    if ([cell isEqual:authorFieldCell]) {
        // Restore the position of the main view if it was animated to make room for the keyboard.
        if  (self.view.frame.origin.y < 0) {
            [self setViewMovedUp:NO];
        }
    }
}

// Animate the entire view up or down, to prevent the keyboard from covering the author field.
- (void)setViewMovedUp:(BOOL)movedUp {
    [UIView beginAnimations:nil context:NULL];
    [UIView setAnimationDuration:0.3];
    // Make changes to the view's frame inside the animation block. They will be animated instead
    // of taking place immediately.
    CGRect rect = self.view.frame;
    if (movedUp) {
        // If moving up, not only decrease the origin but increase the height so the view 
        // covers the entire screen behind the keyboard.
        rect.origin.y -= kOFFSET_FOR_KEYBOARD;
        rect.size.height += kOFFSET_FOR_KEYBOARD;
    } else {
        // If moving down, not only increase the origin but decrease the height.
        rect.origin.y += kOFFSET_FOR_KEYBOARD;
        rect.size.height -= kOFFSET_FOR_KEYBOARD;
    }
    self.view.frame = rect;
    
    [UIView commitAnimations];
}

- (void)setEditing:(BOOL)editing animated:(BOOL)animated {
    [super setEditing:editing animated:animated];
    if (!editing) {
        [titleFieldCell stopEditing];
        [copyrightFieldCell stopEditing];
        [authorFieldCell stopEditing];
    }
}

- (void)keyboardWillShow:(NSNotification *)noti {
    // The keyboard will be shown. If the user is editing the author, adjust the display so that the
    // author field will not be covered by the keyboard.
    if (authorFieldCell.isInlineEditing && self.view.frame.origin.y >= 0) {
        [self setViewMovedUp:YES];
    } else if (!authorFieldCell.isInlineEditing && self.view.frame.origin.y < 0) {
        [self setViewMovedUp:NO];

    }
}

#pragma mark -
#pragma mark <UITableViewDelegate, UITableViewDataSource> Methods

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    // Tabelle enthaelt zwei Zeilen (Task und Datum)
    return 2;
}

- (NSInteger)tableView:(UITableView *)table numberOfRowsInSection:(NSInteger)section {
    // Only one row for each section
    return 1;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath 
        withAvailableCell:(UITableViewCell *)availableCell {
    // Return the appropriate cell for the specified section/row.
    // We have 3 sections, each containing only one row, so we can return the cell based on the section number.
    switch (indexPath.section) {
    	case 0: return titleFieldCell;
    	case 1: return copyrightFieldCell;
    }
    return nil;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
    // Return the displayed title for the specified section.
    switch (section) {
    	case 0: return @"ToDo";
    	case 1: return @"Erledigung bis";
    }
    return nil;
}

- (NSIndexPath *)tableView:(UITableView *)tv willSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    // Never allow selection.
    return nil;
}

@end